@@ -408,6 +408,7 @@ theorems :=
 "EQ_TRANS",EQ_TRANS;
 "EQ_UNIV",EQ_UNIV;
 "ETA_AX",ETA_AX;
+"ETA_ONE",ETA_ONE;
 "EVEN",EVEN;
 "EVEN_ADD",EVEN_ADD;
 "EVEN_AND_ODD",EVEN_AND_ODD;
@@ -467,6 +468,7 @@ theorems :=
 "EXISTS_UNIQUE_DEF",EXISTS_UNIQUE_DEF;
 "EXISTS_UNIQUE_REFL",EXISTS_UNIQUE_REFL;
 "EXISTS_UNIQUE_THM",EXISTS_UNIQUE_THM;
+"EXISTS_UNIQUE_UNIONS_INTERS",EXISTS_UNIQUE_UNIONS_INTERS;
 "EXISTS_UNPAIR_FUN_THM",EXISTS_UNPAIR_FUN_THM;
 "EXISTS_UNPAIR_THM",EXISTS_UNPAIR_THM;
 "EXP",EXP;
@@ -585,6 +587,8 @@ theorems :=
 "FINITE_SUBSET_NUMSEG",FINITE_SUBSET_NUMSEG;
 "FINITE_SUBSET_UNIONS",FINITE_SUBSET_UNIONS;
 "FINITE_SUBSET_UNIONS_CHAIN",FINITE_SUBSET_UNIONS_CHAIN;
+"FINITE_SUBSET_UNIONS_CHAIN_EQ",FINITE_SUBSET_UNIONS_CHAIN_EQ;
+"FINITE_SUBSET_UNIONS_DIRECTED_EQ",FINITE_SUBSET_UNIONS_DIRECTED_EQ;
 "FINITE_SUM_IMAGE",FINITE_SUM_IMAGE;
 "FINITE_SUPPORT",FINITE_SUPPORT;
 "FINITE_SUPPORT_DELTA",FINITE_SUPPORT_DELTA;
@@ -1420,7 +1424,9 @@ theorems :=
 "LENGTH",LENGTH;
 "LENGTH_APPEND",LENGTH_APPEND;
 "LENGTH_EQ_CONS",LENGTH_EQ_CONS;
+"LENGTH_EQ_LIST_OF_SEQ",LENGTH_EQ_LIST_OF_SEQ;
 "LENGTH_EQ_NIL",LENGTH_EQ_NIL;
+"LENGTH_FILTER",LENGTH_FILTER;
 "LENGTH_LIST_OF_SEQ",LENGTH_LIST_OF_SEQ;
 "LENGTH_LIST_OF_SET",LENGTH_LIST_OF_SET;
 "LENGTH_MAP",LENGTH_MAP;
@@ -1462,7 +1468,9 @@ theorems :=
 "LE_SUC_LT",LE_SUC_LT;
 "LE_TRANS",LE_TRANS;
 "LIST_EQ",LIST_EQ;
+"LIST_OF_SEQ",LIST_OF_SEQ;
 "LIST_OF_SEQ_EQ_NIL",LIST_OF_SEQ_EQ_NIL;
+"LIST_OF_SEQ_EQ_SELF",LIST_OF_SEQ_EQ_SELF;
 "LIST_OF_SET_EMPTY",LIST_OF_SET_EMPTY;
 "LIST_OF_SET_PROPERTIES",LIST_OF_SET_PROPERTIES;
 "LIST_OF_SET_SING",LIST_OF_SET_SING;
@@ -1506,6 +1514,7 @@ theorems :=
 "MAP_FST_ZIP",MAP_FST_ZIP;
 "MAP_I",MAP_I;
 "MAP_ID",MAP_ID;
+"MAP_LIST_OF_SEQ",MAP_LIST_OF_SEQ;
 "MAP_REVERSE",MAP_REVERSE;
 "MAP_SND_ZIP",MAP_SND_ZIP;
 "MAP_o",MAP_o;
@@ -2379,6 +2388,7 @@ theorems :=
 "RESTRICTION_UNDEFINED",RESTRICTION_UNDEFINED;
 "RESTRICTION_UNIQUE",RESTRICTION_UNIQUE;
 "RESTRICTION_UNIQUE_ALT",RESTRICTION_UNIQUE_ALT;
+"RESTRICTION_UNIV",RESTRICTION_UNIV;
 "REVERSE",REVERSE;
 "REVERSE_APPEND",REVERSE_APPEND;
 "REVERSE_EQ_EMPTY",REVERSE_EQ_EMPTY;
@@ -2411,6 +2421,7 @@ theorems :=
 "SIMPLE_IMAGE",SIMPLE_IMAGE;
 "SIMPLE_IMAGE_GEN",SIMPLE_IMAGE_GEN;
 "SING",SING;
+"SING_ALT",SING_ALT;
 "SING_GSPEC",SING_GSPEC;
 "SING_SUBSET",SING_SUBSET;
 "SKOLEM_THM",SKOLEM_THM;
@@ -2683,6 +2694,7 @@ theorems :=
 "UNIONS_2",UNIONS_2;
 "UNIONS_DELETE_EMPTY",UNIONS_DELETE_EMPTY;
 "UNIONS_DIFF",UNIONS_DIFF;
+"UNIONS_EQ_INTERS",UNIONS_EQ_INTERS;
 "UNIONS_GSPEC",UNIONS_GSPEC;
 "UNIONS_IMAGE",UNIONS_IMAGE;
 "UNIONS_INSERT",UNIONS_INSERT;
@@ -2715,6 +2727,7 @@ theorems :=
 "UNIQUE_SKOLEM_ALT",UNIQUE_SKOLEM_ALT;
 "UNIQUE_SKOLEM_THM",UNIQUE_SKOLEM_THM;
 "UNIV",UNIV;
+"UNIV_1",UNIV_1;
 "UNIV_GSPEC",UNIV_GSPEC;
 "UNIV_HAS_SIZE_DIMINDEX",UNIV_HAS_SIZE_DIMINDEX;
 "UNIV_NOT_EMPTY",UNIV_NOT_EMPTY;
